@@ -772,11 +772,7 @@ PyArray_Mean(PyArrayObject *self, int axis, int rtype, PyArrayObject *out)
         return NULL;
     }
     if (!out) {
-#if defined(NPY_PY3K)
         ret = PyNumber_TrueDivide(obj1, obj2);
-#else
-        ret = PyNumber_Divide(obj1, obj2);
-#endif
     }
     else {
         ret = PyObject_CallFunction(n_ops.divide, "OOO", out, obj2, out);
@@ -930,14 +926,15 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
         }
     }
 
-    /* NumPy 1.17.0, 2019-02-24 */
-    if (DEPRECATE(
-            "->f->fastclip is deprecated. Use PyUFunc_RegisterLoopForDescr to "
-            "attach a custom loop to np.core.umath.clip, np.minimum, and "
-            "np.maximum") < 0) {
-        return NULL;
-    }
-    /* everything below can be removed once this deprecation completes */
+    /*
+     * NumPy 1.17.0, 2019-02-24
+     * NumPy 1.19.0, 2020-01-15
+     *
+     * Setting `->f->fastclip to anything but NULL has been deprecated in 1.19
+     * the code path below was previously deprecated since 1.17.
+     * (the deprecation moved to registration time instead of execution time)
+     * everything below can be removed once this deprecation completes
+     */
 
     if (func == NULL
         || (min != NULL && !PyArray_CheckAnyScalar(min))
