@@ -18,6 +18,7 @@
 
 #include "npy_config.h"
 #include "npy_pycompat.h"
+#include "ctors.h"
 
 #include "numpy/ufuncobject.h"
 #include "lowlevel_strided_loops.h"
@@ -36,7 +37,7 @@
  * If 'dtype' isn't NULL, this function steals its reference.
  */
 static PyArrayObject *
-allocate_reduce_result(PyArrayObject *arr, npy_bool *axis_flags,
+allocate_reduce_result(PyArrayObject *arr, const npy_bool *axis_flags,
                         PyArray_Descr *dtype, int subok)
 {
     npy_intp strides[NPY_MAXDIMS], stride;
@@ -84,16 +85,17 @@ allocate_reduce_result(PyArrayObject *arr, npy_bool *axis_flags,
  * The return value is a view into 'out'.
  */
 static PyArrayObject *
-conform_reduce_result(int ndim, npy_bool *axis_flags,
+conform_reduce_result(PyArrayObject *in, const npy_bool *axis_flags,
                       PyArrayObject *out, int keepdims, const char *funcname,
                       int need_copy)
 {
-    npy_intp strides[NPY_MAXDIMS], shape[NPY_MAXDIMS];
-    npy_intp *strides_out = PyArray_STRIDES(out);
-    npy_intp *shape_out = PyArray_DIMS(out);
-    int idim, idim_out, ndim_out = PyArray_NDIM(out);
-    PyArray_Descr *dtype;
-    PyArrayObject_fields *ret;
+    /* unpack shape information */
+    int const ndim = PyArray_NDIM(in);
+    npy_intp const *shape_in = PyArray_DIMS(in);
+
+    int const ndim_out = PyArray_NDIM(out);
+    npy_intp const *strides_out = PyArray_STRIDES(out);
+    npy_intp const *shape_out = PyArray_DIMS(out);
 
     /*
      * If the 'keepdims' parameter is true, do a simpler validation and
@@ -108,7 +110,7 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
             return NULL;
         }
 
-        for (idim = 0; idim < ndim; ++idim) {
+        for (int idim = 0; idim < ndim; ++idim) {
             if (axis_flags[idim]) {
                 if (shape_out[idim] != 1) {
                     PyErr_Format(PyExc_ValueError,
@@ -118,6 +120,16 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
                     return NULL;
                 }
             }
+            else {
+                if (shape_out[idim] != shape_in[idim]) {
+                    PyErr_Format(PyExc_ValueError,
+                            "output parameter for reduction operation %s "
+                            "has a non-reduction dimension not equal to "
+                            "the input one.", funcname);
+                    return NULL;
+                }
+            }
+
         }
 
         Py_INCREF(out);
@@ -125,8 +137,9 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
     }
 
     /* Construct the strides and shape */
-    idim_out = 0;
-    for (idim = 0; idim < ndim; ++idim) {
+    npy_intp strides[NPY_MAXDIMS], shape[NPY_MAXDIMS];
+    int idim_out = 0;
+    for (int idim = 0; idim < ndim; ++idim) {
         if (axis_flags[idim]) {
             strides[idim] = 0;
             shape[idim] = 1;
@@ -138,6 +151,13 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
                         "does not have enough dimensions", funcname);
                 return NULL;
             }
+            if (shape_out[idim_out] != shape_in[idim]) {
+                PyErr_Format(PyExc_ValueError,
+                        "output parameter for reduction operation %s "
+                        "has a non-reduction dimension not equal to "
+                        "the input one.", funcname);
+                return NULL;
+            }
             strides[idim] = strides_out[idim_out];
             shape[idim] = shape_out[idim_out];
             ++idim_out;
@@ -152,31 +172,20 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
     }
 
     /* Allocate the view */
-    dtype = PyArray_DESCR(out);
+    PyArray_Descr *dtype = PyArray_DESCR(out);
     Py_INCREF(dtype);
 
-    /* TODO: use PyArray_NewFromDescrAndBase here once multiarray and umath
-     *       are merged
-     */
-    ret = (PyArrayObject_fields *)PyArray_NewFromDescr(
+    PyArrayObject_fields *ret = (PyArrayObject_fields *)PyArray_NewFromDescrAndBase(
             &PyArray_Type, dtype,
             ndim, shape, strides, PyArray_DATA(out),
-            PyArray_FLAGS(out), NULL);
+            PyArray_FLAGS(out), NULL, (PyObject *)out);
     if (ret == NULL) {
         return NULL;
     }
 
-    Py_INCREF(out);
-    if (PyArray_SetBaseObject((PyArrayObject *)ret, (PyObject *)out) < 0) {
-        Py_DECREF(ret);
-        return NULL;
-    }
-
     if (need_copy) {
-        PyArrayObject *ret_copy;
-
-        ret_copy = (PyArrayObject *)PyArray_NewLikeArray(
-            (PyArrayObject *)ret, NPY_ANYORDER, NULL, 0);
+        PyArrayObject *ret_copy = (PyArrayObject *)PyArray_NewLikeArray(
+                (PyArrayObject *)ret, NPY_ANYORDER, NULL, 0);
         if (ret_copy == NULL) {
             Py_DECREF(ret);
             return NULL;
@@ -189,7 +198,6 @@ conform_reduce_result(int ndim, npy_bool *axis_flags,
         }
 
         if (PyArray_SetWritebackIfCopyBase(ret_copy, (PyArrayObject *)ret) < 0) {
-            Py_DECREF(ret);
             Py_DECREF(ret_copy);
             return NULL;
         }
@@ -240,7 +248,7 @@ PyArray_CreateReduceResult(PyArrayObject *operand, PyArrayObject *out,
 
         /* Steal the dtype reference */
         Py_XDECREF(dtype);
-        result = conform_reduce_result(PyArray_NDIM(operand), axis_flags,
+        result = conform_reduce_result(operand, axis_flags,
                                        out, keepdims, funcname, need_copy);
     }
 
@@ -251,7 +259,7 @@ PyArray_CreateReduceResult(PyArrayObject *operand, PyArrayObject *out,
  * Count the number of dimensions selected in 'axis_flags'
  */
 static int
-count_axes(int ndim, npy_bool *axis_flags)
+count_axes(int ndim, const npy_bool *axis_flags)
 {
     int idim;
     int naxes = 0;
@@ -299,7 +307,7 @@ count_axes(int ndim, npy_bool *axis_flags)
 NPY_NO_EXPORT PyArrayObject *
 PyArray_InitializeReduceResult(
                     PyArrayObject *result, PyArrayObject *operand,
-                    npy_bool *axis_flags,
+                    const npy_bool *axis_flags,
                     npy_intp *out_skip_first_count, const char *funcname)
 {
     npy_intp *strides, *shape, shape_orig[NPY_MAXDIMS];
@@ -528,7 +536,9 @@ PyUFunc_ReduceWrapper(PyArrayObject *operand, PyArrayObject *out,
                   NPY_ITER_ALIGNED;
     if (wheremask != NULL) {
         op[2] = wheremask;
-        op_dtypes[2] = PyArray_DescrFromType(NPY_BOOL);
+        /* wheremask is guaranteed to be NPY_BOOL, so borrow its reference */
+        op_dtypes[2] = PyArray_DESCR(wheremask);
+        assert(op_dtypes[2]->type_num == NPY_BOOL);
         if (op_dtypes[2] == NULL) {
             goto fail;
         }
