@@ -1,5 +1,3 @@
-from __future__ import division, absolute_import, print_function
-
 from functools import reduce
 
 import numpy as np
@@ -33,7 +31,7 @@ def eq(v, w, msg=''):
     return result
 
 
-class TestMa(object):
+class TestMa:
 
     def setup(self):
         x = np.array([1., 1., 1., -2., pi/2.0, 4., 5., -10., 10., 1., 2., 3.])
@@ -263,7 +261,7 @@ class TestMa(object):
         m = make_mask(n)
         m2 = make_mask(m)
         assert_(m is m2)
-        m3 = make_mask(m, copy=1)
+        m3 = make_mask(m, copy=True)
         assert_(m is not m3)
 
         x1 = np.arange(5)
@@ -570,7 +568,7 @@ class TestMa(object):
         ott = array([0., 1., 2., 3.], mask=[1, 0, 0, 0])
         assert_(eq(2.0, average(ott, axis=0)))
         assert_(eq(2.0, average(ott, weights=[1., 1., 2., 1.])))
-        result, wts = average(ott, weights=[1., 1., 2., 1.], returned=1)
+        result, wts = average(ott, weights=[1., 1., 2., 1.], returned=True)
         assert_(eq(2.0, result))
         assert_(wts == 4.0)
         ott[:] = masked
@@ -581,7 +579,7 @@ class TestMa(object):
         assert_(eq(average(ott, axis=0), [2.0, 0.0]))
         assert_(average(ott, axis=1)[0] is masked)
         assert_(eq([2., 0.], average(ott, axis=0)))
-        result, wts = average(ott, axis=0, returned=1)
+        result, wts = average(ott, axis=0, returned=True)
         assert_(eq(wts, [1., 0.]))
 
     def test_testAverage2(self):
@@ -622,14 +620,14 @@ class TestMa(object):
 
         a = arange(6)
         b = arange(6) * 3
-        r1, w1 = average([[a, b], [b, a]], axis=1, returned=1)
+        r1, w1 = average([[a, b], [b, a]], axis=1, returned=True)
         assert_equal(shape(r1), shape(w1))
         assert_equal(r1.shape, w1.shape)
-        r2, w2 = average(ones((2, 2, 3)), axis=0, weights=[3, 1], returned=1)
+        r2, w2 = average(ones((2, 2, 3)), axis=0, weights=[3, 1], returned=True)
         assert_equal(shape(w2), shape(r2))
-        r2, w2 = average(ones((2, 2, 3)), returned=1)
+        r2, w2 = average(ones((2, 2, 3)), returned=True)
         assert_equal(shape(w2), shape(r2))
-        r2, w2 = average(ones((2, 2, 3)), weights=ones((2, 2, 3)), returned=1)
+        r2, w2 = average(ones((2, 2, 3)), weights=ones((2, 2, 3)), returned=True)
         assert_(shape(w2) == shape(r2))
         a2d = array([[1, 2], [0, 4]], float)
         a2dm = masked_array(a2d, [[0, 0], [1, 0]])
@@ -700,7 +698,7 @@ class TestMa(object):
         assert_equal(b[1].shape, ())
 
 
-class TestUfuncs(object):
+class TestUfuncs:
     def setup(self):
         self.d = (array([1.0, 0, -1, pi / 2] * 2, mask=[0, 1] + [0] * 6),
                   array([1.0, 0, -1, pi / 2] * 2, mask=[1, 0] + [0] * 6),)
@@ -765,7 +763,7 @@ class TestUfuncs(object):
             assert_(eq(nonzero(x), [0]))
 
 
-class TestArrayMethods(object):
+class TestArrayMethods:
 
     def setup(self):
         x = np.array([8.375, 7.545, 8.828, 8.5, 1.757, 5.928,
